# #########################################################################
# Copyright (c) 2020, UChicago Argonne, LLC. All rights reserved.         #
#                                                                         #
# Copyright 2020. UChicago Argonne, LLC. This software was produced       #
# under U.S. Government contract DE-AC02-06CH11357 for Argonne National   #
# Laboratory (ANL), which is operated by UChicago Argonne, LLC for the    #
# U.S. Department of Energy. The U.S. Government has rights to use,       #
# reproduce, and distribute this software.  NEITHER THE GOVERNMENT NOR    #
# UChicago Argonne, LLC MAKES ANY WARRANTY, EXPRESS OR IMPLIED, OR        #
# ASSUMES ANY LIABILITY FOR THE USE OF THIS SOFTWARE.  If software is     #
# modified to produce derivative works, such modified software should     #
# be clearly marked, so as not to confuse it with the version available   #
# from ANL.                                                               #
#                                                                         #
# Additionally, redistribution and use in source and binary forms, with   #
# or without modification, are permitted provided that the following      #
# conditions are met:                                                     #
#                                                                         #
#     * Redistributions of source code must retain the above copyright    #
#       notice, this list of conditions and the following disclaimer.     #
#                                                                         #
#     * Redistributions in binary form must reproduce the above copyright #
#       notice, this list of conditions and the following disclaimer in   #
#       the documentation and/or other materials provided with the        #
#       distribution.                                                     #
#                                                                         #
#     * Neither the name of UChicago Argonne, LLC, Argonne National       #
#       Laboratory, ANL, the U.S. Government, nor the names of its        #
#       contributors may be used to endorse or promote products derived   #
#       from this software without specific prior written permission.     #
#                                                                         #
# THIS SOFTWARE IS PROVIDED BY UChicago Argonne, LLC AND CONTRIBUTORS     #
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT       #
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS       #
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL UChicago     #
# Argonne, LLC OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,        #
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,    #
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;        #
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER        #
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT      #
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN       #
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE         #
# POSSIBILITY OF SUCH DAMAGE.                                             #
# #########################################################################
import os

from PyQt5.QtCore import QSettings
from PyQt5.QtGui import QPalette, QColor, QFont

from orangewidget import gui

from orangecontrib.wavepy2.util.gui.ow_wavepy_widget import WavePyWidget
from orangecontrib.wavepy2.util.wavepy_objects import OasysWavePyData, register_log_stream_widget_instance, get_registered_log_stream_instance

from aps.wavepy2.util import AlreadyInitializedError
from aps.util.logger import register_logger_single_instance, LoggerMode
from aps.wavepy2.util import register_plotter_instance, PlotterMode
from aps.wavepy2.util import PlottingProperties
from aps.util.initializer import register_ini_instance, get_registered_ini_instance, IniMode

class WavePyInitWidget(WavePyWidget):
    outputs = [{"name": "WavePy Data",
                "type": OasysWavePyData,
                "doc": "WavePy Data",
                "id": "WavePy_Data"}]

    LOG_STDOUT = False if not "LOGSTDOUT" in os.environ.keys() else str(os.environ.get('LOGSTDOUT')) == "1"

    CONTROL_AREA_WIDTH = 860

    MAX_WIDTH_NO_MAIN = CONTROL_AREA_WIDTH + 10
    MAX_HEIGHT = 660

    def __init__(self, show_general_option_box=False):
        super(WavePyInitWidget, self).__init__(show_general_option_box=show_general_option_box, show_automatic_box=False)

        self._is_valid_widget = True

        try:
            self.initialize_business_logic()

            self._init_widget     = self._draw_init_widget()

            self.controlArea.setFixedHeight(self._init_widget.height() + 145)
            self.setFixedWidth(self.MAX_WIDTH_NO_MAIN)
            self.setFixedHeight(self.MAX_HEIGHT)
        except Exception as e:
            self._is_valid_widget = False

            label = gui.label(self._wavepy_widget_area, self, label="\n\n    INVALID WIDGET:\n\n    " + str(e))

            font = QFont(label.font())
            font.setBold(True)
            font.setPointSize(24)
            label.setFont(font)
            palette = QPalette(label.palette())
            palette.setColor(QPalette.Text, QColor('dark blue'))
            palette.setColor(QPalette.Base, QColor(243, 240, 160))
            label.setPalette(palette)

            self.controlArea.setFixedHeight(300)
            self.setFixedWidth(self.MAX_WIDTH_NO_MAIN)
            self.setFixedHeight(320)

            if self.IS_DEVELOP: raise e

        gui.rubber(self.controlArea)

    ##################################################################################
    # INITIALIZATION

    def initialize_business_logic(self):
        self.initialize_logger()
        self.initialize_plotter()
        self.initialize_ini()
        self.initialize_process_manager()

    def initialize_ini(self):
        try:
            register_ini_instance(IniMode.LOCAL_FILE, ini_file_name=self._get_file_ini_name(), application_name=self._get_application_name())
        except AlreadyInitializedError:
            if not get_registered_ini_instance(application_name=self._get_application_name()).get_ini_file_name() == self._get_file_ini_name():
                raise ValueError("The Oasys worspace can contain only 1 kind of analysis at a time")

    def initialize_logger(self, replace=False):
        try: register_log_stream_widget_instance(application_name=self._get_application_name())
        except AlreadyInitializedError: pass

        if not self.LOG_STDOUT:
            try: register_logger_single_instance(logger_mode=QSettings().value("wavepy/logger_mode", LoggerMode.FULL, type=int),
                                                 stream=get_registered_log_stream_instance(application_name=self._get_application_name()),
                                                 application_name=self._get_application_name(), replace=replace)
            except AlreadyInitializedError: pass
        else:
            try: register_logger_single_instance(logger_mode=QSettings().value("wavepy/logger_mode", LoggerMode.FULL, type=int),
                                                 application_name=self._get_application_name(), replace=replace)
            except AlreadyInitializedError: pass

    def initialize_plotter(self, replace=False):
        try: register_plotter_instance(plotter_mode=QSettings().value("wavepy/plotter_mode", PlotterMode.FULL, type=int),
                                       application_name=self._get_application_name(), replace=replace)
        except AlreadyInitializedError: pass

    def initialize_process_manager(self):
        self._process_manager = self._create_process_manager()

    ##################################################################################
    # ABSTRACT METHODS

    def _get_application_name(self):
        return None

    def _get_file_ini_name(self):
        raise NotImplementedError()

    def _create_process_manager(self):
        raise NotImplementedError()

    def _draw_init_widget(self):
        raise NotImplementedError()

    ##################################################################################
    # DEFAULT METHODS

    def _execute(self):
        if self._is_valid_widget:
            output = OasysWavePyData()

            output.set_process_manager(self._process_manager)
            output.set_initialization_parameters(self._init_widget.get_accepted_output())

            self.send("WavePy Data", output)

    def _get_default_plotting_properties(self):
        return PlottingProperties(context_widget=self._get_default_context(),
                                  show_runtime_options=False,
                                  add_context_label=False,
                                  use_unique_id=True)



# #########################################################################
# Copyright (c) 2020, UChicago Argonne, LLC. All rights reserved.         #
#                                                                         #
# Copyright 2020. UChicago Argonne, LLC. This software was produced       #
# under U.S. Government contract DE-AC02-06CH11357 for Argonne National   #
# Laboratory (ANL), which is operated by UChicago Argonne, LLC for the    #
# U.S. Department of Energy. The U.S. Government has rights to use,       #
# reproduce, and distribute this software.  NEITHER THE GOVERNMENT NOR    #
# UChicago Argonne, LLC MAKES ANY WARRANTY, EXPRESS OR IMPLIED, OR        #
# ASSUMES ANY LIABILITY FOR THE USE OF THIS SOFTWARE.  If software is     #
# modified to produce derivative works, such modified software should     #
# be clearly marked, so as not to confuse it with the version available   #
# from ANL.                                                               #
#                                                                         #
# Additionally, redistribution and use in source and binary forms, with   #
# or without modification, are permitted provided that the following      #
# conditions are met:                                                     #
#                                                                         #
#     * Redistributions of source code must retain the above copyright    #
#       notice, this list of conditions and the following disclaimer.     #
#                                                                         #
#     * Redistributions in binary form must reproduce the above copyright #
#       notice, this list of conditions and the following disclaimer in   #
#       the documentation and/or other materials provided with the        #
#       distribution.                                                     #
#                                                                         #
#     * Neither the name of UChicago Argonne, LLC, Argonne National       #
#       Laboratory, ANL, the U.S. Government, nor the names of its        #
#       contributors may be used to endorse or promote products derived   #
#       from this software without specific prior written permission.     #
#                                                                         #
# THIS SOFTWARE IS PROVIDED BY UChicago Argonne, LLC AND CONTRIBUTORS     #
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT       #
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS       #
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL UChicago     #
# Argonne, LLC OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,        #
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,    #
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;        #
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER        #
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT      #
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN       #
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE         #
# POSSIBILITY OF SUCH DAMAGE.                                             #
# #########################################################################
from multiprocessing import cpu_count

from orangewidget.settings import Setting
from orangewidget import gui
from oasys.widgets import gui as oasysgui

from orangecontrib.wavepy2.util.gui.ow_wavepy_init_widget import WavePyInitWidget

from aps.wavepy2.tools.diagnostic.coherence.bl.single_grating_coherence_z_scan import create_single_grating_coherence_z_scan_manager, SINGLE_THREAD, APPLICATION_NAME

N_CPUS = cpu_count() - 2

class OWSGZInit(WavePyInitWidget):
    name = "S.G.Z. - Initialization"
    id = "sgz_init"
    description = "S.G.Z. - Initialization"
    icon = "icons/sgz_init.png"
    priority = 1
    category = ""
    keywords = ["wavepy", "sgz", "init"]

    parallel_mode = Setting(1)

    MAX_HEIGHT = 700

    def __init__(self):
        super(OWSGZInit, self).__init__(show_general_option_box=True)

        self.__draw_additional_input_area()

    def _get_application_name(self):
        return APPLICATION_NAME

    def _get_file_ini_name(self):
        return ".single_grating_coherence_z_scan.ini"

    def _create_process_manager(self):
        if N_CPUS > 1: return create_single_grating_coherence_z_scan_manager(mode=self.parallel_mode)
        else:          return create_single_grating_coherence_z_scan_manager(mode=SINGLE_THREAD)

    def __draw_additional_input_area(self):
        cb_parallel_mode = gui.comboBox(oasysgui.widgetBox(self._general_options_box, "", addSpace=False, orientation="vertical", width=350), self,
                                        "parallel_mode", items=["Single-Thread", "Multi-Thread"], label="Computing Mode",
                                        labelWidth=200, orientation="horizontal", callback=self.__set_parallel_mode)

        if N_CPUS < 2:
            self.parallel_mode = SINGLE_THREAD
            cb_parallel_mode.setReadOnly(True)
        else:
            self.__cpus_label = gui.label(oasysgui.widgetBox(self._general_options_box, "", addSpace=False, orientation="vertical"), self,
                                          label=("  # cpus: " + str(N_CPUS)))
            self.__set_parallel_mode()

        self.controlArea.setFixedHeight(self._init_widget.height() + 195)
        gui.rubber(self.controlArea)

    def _draw_init_widget(self):
        return self._process_manager.draw_initialization_parameters_widget(plotting_properties=self._get_default_plotting_properties(),
                                                                           widget_height=485,
                                                                           tab_widget_width=self.CONTROL_AREA_WIDTH-20)[0]

    def _execute(self):
        self._process_manager = self._create_process_manager()

        super(OWSGZInit, self)._execute()

    def __set_parallel_mode(self):
        self.__cpus_label.setVisible(self.parallel_mode==1)



# #########################################################################
# Copyright (c) 2020, UChicago Argonne, LLC. All rights reserved.         #
#                                                                         #
# Copyright 2020. UChicago Argonne, LLC. This software was produced       #
# under U.S. Government contract DE-AC02-06CH11357 for Argonne National   #
# Laboratory (ANL), which is operated by UChicago Argonne, LLC for the    #
# U.S. Department of Energy. The U.S. Government has rights to use,       #
# reproduce, and distribute this software.  NEITHER THE GOVERNMENT NOR    #
# UChicago Argonne, LLC MAKES ANY WARRANTY, EXPRESS OR IMPLIED, OR        #
# ASSUMES ANY LIABILITY FOR THE USE OF THIS SOFTWARE.  If software is     #
# modified to produce derivative works, such modified software should     #
# be clearly marked, so as not to confuse it with the version available   #
# from ANL.                                                               #
#                                                                         #
# Additionally, redistribution and use in source and binary forms, with   #
# or without modification, are permitted provided that the following      #
# conditions are met:                                                     #
#                                                                         #
#     * Redistributions of source code must retain the above copyright    #
#       notice, this list of conditions and the following disclaimer.     #
#                                                                         #
#     * Redistributions in binary form must reproduce the above copyright #
#       notice, this list of conditions and the following disclaimer in   #
#       the documentation and/or other materials provided with the        #
#       distribution.                                                     #
#                                                                         #
#     * Neither the name of UChicago Argonne, LLC, Argonne National       #
#       Laboratory, ANL, the U.S. Government, nor the names of its        #
#       contributors may be used to endorse or promote products derived   #
#       from this software without specific prior written permission.     #
#                                                                         #
# THIS SOFTWARE IS PROVIDED BY UChicago Argonne, LLC AND CONTRIBUTORS     #
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT       #
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS       #
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL UChicago     #
# Argonne, LLC OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,        #
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,    #
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;        #
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER        #
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT      #
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN       #
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE         #
# POSSIBILITY OF SUCH DAMAGE.                                             #
# #########################################################################
from oasys.widgets.widget import OWWidget
from orangewidget import gui
from oasys.widgets import gui as oasysgui
from orangewidget.widget import OWAction
from orangewidget.settings import Setting

from aps.wavepy2.util import DefaultContextWidget, PlottingProperties

from PyQt5.QtWidgets import QApplication
from PyQt5.QtCore import QRect

INITIALIZATION_PARAMETERS = "initialization_parameters"
CALCULATION_PARAMETERS    = "calculation_parameters"
PROCESS_MANAGER           = "process_manager"

class WavePyWidget(OWWidget):
    want_main_area = 0

    is_automatic_run = Setting(True)

    error_id = 0
    warning_id = 0
    info_id = 0

    CONTROL_AREA_WIDTH = 405

    MAX_WIDTH_FULL = 1320
    MAX_WIDTH_NO_MAIN = CONTROL_AREA_WIDTH + 10
    MAX_HEIGHT = 700

    def __init__(self, show_general_option_box=False, show_automatic_box=False):
        super().__init__()

        runaction = OWAction(self._get_execute_button_label(), self)
        runaction.triggered.connect(self._execute)
        self.addAction(runaction)

        geom = QApplication.desktop().availableGeometry()
        self.setGeometry(QRect(round(geom.width()*0.05),
                               round(geom.height()*0.05),
                               round(min(geom.width() * 0.98, self.MAX_WIDTH_FULL if self.want_main_area==1 else self.MAX_WIDTH_NO_MAIN)),
                               round(min(geom.height()*0.95, self.MAX_HEIGHT))))

        self.setMaximumHeight(self.geometry().height())
        self.setMaximumWidth(self.geometry().width())

        self.controlArea.setFixedWidth(self.CONTROL_AREA_WIDTH)

        self._general_options_box = oasysgui.widgetBox(self.controlArea, "General Options", addSpace=True, orientation="horizontal")
        self._general_options_box.setVisible(show_general_option_box)

        if show_automatic_box :
            gui.checkBox(self._general_options_box, self, 'is_automatic_run', 'Automatic Execution')

        self._button_box = oasysgui.widgetBox(self.controlArea, "", addSpace=False, orientation="horizontal")
        gui.button(self._button_box, self, self._get_execute_button_label(), callback=self._execute, height=45)

        self._wavepy_widget_area = oasysgui.widgetBox(self.controlArea, "", addSpace=False, orientation="horizontal", width=self.CONTROL_AREA_WIDTH)

    def _execute(self):
        raise NotImplementedError("This method is abstract")

    def _get_execute_button_label(self):
        return "Execute"

    def _clear_wavepy_layout(self):
        clear_layout(self._wavepy_widget_area.layout())

    def _get_default_context(self):
        return DefaultContextWidget(self._wavepy_widget_area)

    def _get_default_plotting_properties(self):
        return PlottingProperties(context_widget=self._get_default_context(),
                                  add_context_label=False,
                                  use_unique_id=True)

def clear_layout(layout):
    for i in reversed(range(layout.count())):
        layoutItem = layout.itemAt(i)
        if layoutItem.widget() is not None:
            widgetToRemove = layoutItem.widget()
            widgetToRemove.setParent(None)
            layout.removeWidget(widgetToRemove)
        else:
            layoutToRemove = layout.itemAt(i)
            clear_layout(layoutToRemove)



# #########################################################################
# Copyright (c) 2020, UChicago Argonne, LLC. All rights reserved.         #
#                                                                         #
# Copyright 2020. UChicago Argonne, LLC. This software was produced       #
# under U.S. Government contract DE-AC02-06CH11357 for Argonne National   #
# Laboratory (ANL), which is operated by UChicago Argonne, LLC for the    #
# U.S. Department of Energy. The U.S. Government has rights to use,       #
# reproduce, and distribute this software.  NEITHER THE GOVERNMENT NOR    #
# UChicago Argonne, LLC MAKES ANY WARRANTY, EXPRESS OR IMPLIED, OR        #
# ASSUMES ANY LIABILITY FOR THE USE OF THIS SOFTWARE.  If software is     #
# modified to produce derivative works, such modified software should     #
# be clearly marked, so as not to confuse it with the version available   #
# from ANL.                                                               #
#                                                                         #
# Additionally, redistribution and use in source and binary forms, with   #
# or without modification, are permitted provided that the following      #
# conditions are met:                                                     #
#                                                                         #
#     * Redistributions of source code must retain the above copyright    #
#       notice, this list of conditions and the following disclaimer.     #
#                                                                         #
#     * Redistributions in binary form must reproduce the above copyright #
#       notice, this list of conditions and the following disclaimer in   #
#       the documentation and/or other materials provided with the        #
#       distribution.                                                     #
#                                                                         #
#     * Neither the name of UChicago Argonne, LLC, Argonne National       #
#       Laboratory, ANL, the U.S. Government, nor the names of its        #
#       contributors may be used to endorse or promote products derived   #
#       from this software without specific prior written permission.     #
#                                                                         #
# THIS SOFTWARE IS PROVIDED BY UChicago Argonne, LLC AND CONTRIBUTORS     #
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT       #
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS       #
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL UChicago     #
# Argonne, LLC OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,        #
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,    #
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;        #
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER        #
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT      #
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN       #
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE         #
# POSSIBILITY OF SUCH DAMAGE.                                             #
# #########################################################################
import copy

class OasysWavePyData(object):
    def __init__(self, process_manager=None, initialization_parameters=None, calculation_parameters=None, **parameters):
        self.__process_manager = process_manager
        self.__initialization_parameters = initialization_parameters
        self.__calculation_parameters = calculation_parameters
        self.__parameters = parameters

    def get_process_manager(self):
        return self.__process_manager

    def set_process_manager(self, process_manager=None):
        self.__process_manager = process_manager

    def get_initialization_parameters(self):
        return self.__initialization_parameters

    def set_initialization_parameters(self, initialization_parameters=None):
        self.__initialization_parameters = initialization_parameters

    def get_calculation_parameters(self):
        return self.__calculation_parameters

    def set_calculation_parameters(self, calculation_parameters=None):
        self.__calculation_parameters = calculation_parameters

    def get_parameters(self):
        return self.__parameters

    def get_parameter(self, parameter_name, default_value=None):
        try:
            return self.__parameters[parameter_name]
        except:
            return default_value

    def set_parameter(self, parameter_name, value):
        self.__parameters[parameter_name] = value

    def duplicate(self):
        duplicated = OasysWavePyData(self.__process_manager,
                                     None if self.__initialization_parameters is None else self.__initialization_parameters.duplicate(),
                                     None if self.__calculation_parameters is None else self.__calculation_parameters.duplicate())

        if not self.__parameters is None:
            for parameter_name in self.__parameters.keys():
                duplicated.set_parameter(parameter_name, copy.deepcopy(self.get_parameter(parameter_name)))

        return duplicated

from PyQt5.QtWidgets import QWidget
from PyQt5.Qt import QTextCursor


from aps.util.logger import LogStream

import oasys.widgets.gui as gui

class LogStreamWidget(LogStream):
    class Widget(QWidget):
        def __init__(self, width=850, height=400):
            QWidget.__init__(self)

            self.setFixedHeight(height)
            self.setFixedWidth(width)

            text_area_box = gui.widgetBox(self, "", orientation="vertical", height=height, width=width)

            self.__text_area = gui.textArea(height=height-5, width=width-5, readOnly=True)
            self.__text_area.setText("")

            text_area_box.layout().addWidget(self.__text_area)

        def write(self, text):
            cursor = self.__text_area.textCursor()
            cursor.movePosition(QTextCursor.End)
            cursor.insertText(text)
            self.__text_area.setTextCursor(cursor)
            self.__text_area.ensureCursorVisible()

        def clear_log(self):
            self.__text_area.clear()

    def __init__(self, width=850, height=400):
        self.__widget = LogStreamWidget.Widget(width, height)

    def close(self): pass
    def write(self, text): self.__widget.write(text)
    def flush(self, *args, **kwargs): pass

    def get_widget(self):
        return self.__widget

from aps.wavepy2.util import Singleton, synchronized_method
from aps.wavepy2.util import GenericRegistry

@Singleton
class __LogStreamRegistry(GenericRegistry):
    def __init__(self):
        GenericRegistry.__init__(self, registry_name="Log Stream")

    @synchronized_method
    def register_log_stream(self, log_stream, application_name=None, replace=False):
        super().register_instance(log_stream, application_name, replace)

    @synchronized_method
    def reset(self, application_name=None):
        super().reset(application_name)

    def get_log_stream_instance(self, application_name=None):
        return super().get_instance(application_name)

def register_log_stream_widget_instance(application_name=None, reset=False, replace=False):
    if reset: __LogStreamRegistry.Instance().reset(application_name)
    __LogStreamRegistry.Instance().register_log_stream(LogStreamWidget(width=1095, height=450), application_name, replace)

def get_registered_log_stream_instance(application_name=None):
    return __LogStreamRegistry.Instance().get_log_stream_instance(application_name)



# #########################################################################
# Copyright (c) 2020, UChicago Argonne, LLC. All rights reserved.         #
#                                                                         #
# Copyright 2020. UChicago Argonne, LLC. This software was produced       #
# under U.S. Government contract DE-AC02-06CH11357 for Argonne National   #
# Laboratory (ANL), which is operated by UChicago Argonne, LLC for the    #
# U.S. Department of Energy. The U.S. Government has rights to use,       #
# reproduce, and distribute this software.  NEITHER THE GOVERNMENT NOR    #
# UChicago Argonne, LLC MAKES ANY WARRANTY, EXPRESS OR IMPLIED, OR        #
# ASSUMES ANY LIABILITY FOR THE USE OF THIS SOFTWARE.  If software is     #
# modified to produce derivative works, such modified software should     #
# be clearly marked, so as not to confuse it with the version available   #
# from ANL.                                                               #
#                                                                         #
# Additionally, redistribution and use in source and binary forms, with   #
# or without modification, are permitted provided that the following      #
# conditions are met:                                                     #
#                                                                         #
#     * Redistributions of source code must retain the above copyright    #
#       notice, this list of conditions and the following disclaimer.     #
#                                                                         #
#     * Redistributions in binary form must reproduce the above copyright #
#       notice, this list of conditions and the following disclaimer in   #
#       the documentation and/or other materials provided with the        #
#       distribution.                                                     #
#                                                                         #
#     * Neither the name of UChicago Argonne, LLC, Argonne National       #
#       Laboratory, ANL, the U.S. Government, nor the names of its        #
#       contributors may be used to endorse or promote products derived   #
#       from this software without specific prior written permission.     #
#                                                                         #
# THIS SOFTWARE IS PROVIDED BY UChicago Argonne, LLC AND CONTRIBUTORS     #
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT       #
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS       #
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL UChicago     #
# Argonne, LLC OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,        #
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,    #
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;        #
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER        #
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT      #
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN       #
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE         #
# POSSIBILITY OF SUCH DAMAGE.                                             #
# #########################################################################
from PyQt5.QtWidgets import QMessageBox
from PyQt5.QtCore import QSettings

from orangecanvas.scheme.link import SchemeLink
from oasys.menus.menu import OMenu

from aps.wavepy2.util import PlotterMode
from aps.util.logger import LoggerMode

from orangecontrib.wavepy2.util.gui.ow_wavepy_init_widget import WavePyInitWidget

base_tools_path = "orangecontrib.wavepy2.widgets.tools."
base_imaging_path = "orangecontrib.wavepy2.widgets.imaging."
base_diagnostic_path = "orangecontrib.wavepy2.widgets.diagnostic."
base_metrology_path = "orangecontrib.wavepy2.widgets.metrology."

sgt_analysis_widget_list_not_interactive = [
    [base_imaging_path + "ow_sgt_init.OWSGTInit", (0.0, 50.0), {}],
    [base_imaging_path + "ow_sgt_manager_initialization.OWSGTManagerInitialization", None, {"is_automatic_run": True}],
    [base_imaging_path + "ow_sgt_crop_initial_image.OWSGTCropInitialImage", None, {"is_automatic_run": True}],
    [base_imaging_path + "ow_sgt_crop_reference_image.OWSGTCropReferenceImage", None, {"is_automatic_run": True}],
    [base_imaging_path + "ow_sgt_calculate_dpc.OWSGTCalculateDPC", (50.0, 200.0), {"is_automatic_run": True, "show_results_when_ready" : False}],
    [base_imaging_path + "ow_sgt_crop_calculated_dpc.OWSGTCropCalculatedDPC", None, {"is_automatic_run": True}],
    [base_imaging_path + "ow_sgt_show_calculated_dpc.OWSGTShowCalculatedDPC", None, {"is_automatic_run": True, "show_results_when_ready" : False}],
    [base_imaging_path + "ow_sgt_correct_zero_dpc.OWSGTCorrectZeroDPC", None, {"is_automatic_run": True, "show_results_when_ready" : False, "correct_dpc_center": 0}],
    [base_imaging_path + "ow_sgt_remove_linear_fit_dpc.OWSGTRemoveLinearFitDPC", None, {"is_automatic_run": True, "show_results_when_ready" : False}],
    [base_imaging_path + "ow_sgt_dpc_profile_analysis.OWSGTDPCProfileAnalysis", None, {"is_automatic_run": True, "show_results_when_ready" : False}],
    [base_imaging_path + "ow_sgt_fit_radius_dpc.OWSGTFitRadiusDPC", None, {"is_automatic_run": True, "show_results_when_ready" : False}],
    [base_imaging_path + "ow_sgt_crop_dpc_for_integration.OWSGTCropDPCForIntegration", (50.0, 350.0), {"is_automatic_run": True}],
    [base_imaging_path + "ow_sgt_do_integration.OWSGTDoIntegration", None, {"is_automatic_run": True, "show_results_when_ready" : False}],
    [base_imaging_path + "ow_sgt_calculate_thickness.OWSGTCalculateThickness", None, {"is_automatic_run": True, "show_results_when_ready" : False}],
    [base_imaging_path + "ow_sgt_crop_2nd_order_component_of_the_phase_1.OWSGTCrop2ndOrderComponentOfThePhase1", None, {"is_automatic_run": True}],
    [base_imaging_path + "ow_sgt_calculate_2nd_order_component_of_the_phase_1.OWSGTCalculate2ndOrderComponentOfThePhase1", None, {"is_automatic_run": True, "show_results_when_ready" : False}],
    [base_imaging_path + "ow_sgt_crop_2nd_order_component_of_the_phase_2.OWSGTCrop2ndOrderComponentOfThePhase2", None, {"is_automatic_run": True}],
    [base_imaging_path + "ow_sgt_calculate_2nd_order_component_of_the_phase_2.OWSGTCalculate2ndOrderComponentOfThePhase2", None, {"is_automatic_run": True, "show_results_when_ready" : False}],
    [base_imaging_path + "ow_sgt_remove_2nd_order.OWSGTRemove2ndOrder", None, {"is_automatic_run": True, "show_results_when_ready" : False}],
]

sgt_analysis_widget_list_interactive = [
    [base_imaging_path + "ow_sgt_init.OWSGTInit", (0.0, 50.0), {}],
    [base_imaging_path + "ow_sgt_manager_initialization.OWSGTManagerInitialization", None, {"is_automatic_run": True}],
    [base_imaging_path + "ow_sgt_crop_initial_image.OWSGTCropInitialImage", None, {"is_automatic_run": False}],
    [base_imaging_path + "ow_sgt_crop_reference_image.OWSGTCropReferenceImage", None, {"is_automatic_run": True}],
    [base_imaging_path + "ow_sgt_calculate_dpc.OWSGTCalculateDPC", (50.0, 200.0), {"is_automatic_run": True}],
    [base_imaging_path + "ow_sgt_crop_calculated_dpc.OWSGTCropCalculatedDPC", None, {"is_automatic_run": False}],
    [base_imaging_path + "ow_sgt_show_calculated_dpc.OWSGTShowCalculatedDPC", None, {"is_automatic_run": True}],
    [base_imaging_path + "ow_sgt_correct_zero_dpc.OWSGTCorrectZeroDPC", None, {"is_automatic_run": True, "correct_dpc_center": 1}],
    [base_imaging_path + "ow_sgt_remove_linear_fit_dpc.OWSGTRemoveLinearFitDPC", None, {"is_automatic_run": True}],
    [base_imaging_path + "ow_sgt_dpc_profile_analysis.OWSGTDPCProfileAnalysis", None, {"is_automatic_run": True}],
    [base_imaging_path + "ow_sgt_fit_radius_dpc.OWSGTFitRadiusDPC", None, {"is_automatic_run": True}],
    [base_imaging_path + "ow_sgt_crop_dpc_for_integration.OWSGTCropDPCForIntegration", (50.0, 350.0), {"is_automatic_run": False}],
    [base_imaging_path + "ow_sgt_do_integration.OWSGTDoIntegration", None, {"is_automatic_run": True}],
    [base_imaging_path + "ow_sgt_calculate_thickness.OWSGTCalculateThickness", None, {"is_automatic_run": True}],
    [base_imaging_path + "ow_sgt_crop_2nd_order_component_of_the_phase_1.OWSGTCrop2ndOrderComponentOfThePhase1", None, {"is_automatic_run": False}],
    [base_imaging_path + "ow_sgt_calculate_2nd_order_component_of_the_phase_1.OWSGTCalculate2ndOrderComponentOfThePhase1", None, {"is_automatic_run": True}],
    [base_imaging_path + "ow_sgt_crop_2nd_order_component_of_the_phase_2.OWSGTCrop2ndOrderComponentOfThePhase2", None, {"is_automatic_run": False}],
    [base_imaging_path + "ow_sgt_calculate_2nd_order_component_of_the_phase_2.OWSGTCalculate2ndOrderComponentOfThePhase2", None, {"is_automatic_run": True}],
    [base_imaging_path + "ow_sgt_remove_2nd_order.OWSGTRemove2ndOrder", None, {"is_automatic_run": True}],
]

sgt_analysis_logger_widget_props = [base_imaging_path + "ow_sgt_logger.OWSGTLogger", (0.0, 500.0), {}]

sgz_analysis_widget_list_not_interactive = [
    [base_diagnostic_path + "ow_sgz_init.OWSGZInit", (0.0, 50.0), {}],
    [base_diagnostic_path + "ow_sgz_manager_initialization.OWSGZManagerInitialization", None, {"is_automatic_run": True}],
    [base_diagnostic_path + "ow_sgz_crop_initial_image.OWSGZCropInitialImage", None, {"is_automatic_run": True}],
    [base_diagnostic_path + "ow_sgz_crop_dark_image.OWSGZCropDarkImage", None, {"is_automatic_run": True}],
    [base_diagnostic_path + "ow_sgz_calculate_harmonic_periods.OWSGZCalculateHarmonicPeriods", (50.0, 200.0), {"is_automatic_run": True, "show_results_when_ready" : False}],
    [base_diagnostic_path + "ow_sgz_run_calculation.OWSGZRunCalculation", None, {"is_automatic_run": True, "show_fourier" : 0}],
    [base_diagnostic_path + "ow_sgz_fit_period.OWSGZFitPeriod", None, {"is_automatic_run": True, "show_results_when_ready" : False}],
    [base_diagnostic_path + "ow_sgz_fit_visibility.OWSGZFitVisibility", None, {"is_automatic_run": True, "show_results_when_ready" : False}],
]

sgz_analysis_widget_list_interactive = [
    [base_diagnostic_path + "ow_sgz_init.OWSGZInit", (0.0, 50.0), {}],
    [base_diagnostic_path + "ow_sgz_manager_initialization.OWSGZManagerInitialization", None, {"is_automatic_run": True}],
    [base_diagnostic_path + "ow_sgz_crop_initial_image.OWSGZCropInitialImage", None, {"is_automatic_run": False}],
    [base_diagnostic_path + "ow_sgz_crop_dark_image.OWSGZCropDarkImage", None, {"is_automatic_run": False}],
    [base_diagnostic_path + "ow_sgz_calculate_harmonic_periods.OWSGZCalculateHarmonicPeriods", (50.0, 200.0), {"is_automatic_run": True}],
    [base_diagnostic_path + "ow_sgz_run_calculation.OWSGZRunCalculation", None, {"is_automatic_run": True, "show_fourier" : 0}],
    [base_diagnostic_path + "ow_sgz_fit_period.OWSGZFitPeriod", None, {"is_automatic_run": True}],
    [base_diagnostic_path + "ow_sgz_fit_visibility.OWSGZFitVisibility", None, {"is_automatic_run": True}],
]

sgz_analysis_logger_widget_props = [base_diagnostic_path + "ow_sgz_logger.OWSGZLogger", (0.0, 350.0), {}]

frl_analysis_widget_list_not_interactive = [
    [base_metrology_path + "ow_frl_init.OWFRLInit", (0.0, 50.0), {}],
    [base_metrology_path + "ow_frl_manager_initialization.OWFRLManagerInitialization", None, {"is_automatic_run": True}],
    [base_metrology_path + "ow_frl_crop_thickness.OWFRLCropThickness", None, {"is_automatic_run": True}],
    [base_metrology_path + "ow_frl_manage_crop_thickness.OWFRLManageCropThickness", None, {"is_automatic_run": True, "show_results_when_ready" : False}],
    [base_metrology_path + "ow_frl_center_image.OWFRLCenterImage", (50.0, 200.0), {"is_automatic_run": True, "show_results_when_ready" : False}],
    [base_metrology_path + "ow_frl_fit_radius_dpc.OWFRLFitRadiusDPC", None, {"is_automatic_run": True, "show_results_when_ready" : False}],
    [base_metrology_path + "ow_frl_do_fit.OWFRLDoFit", None, {"is_automatic_run": True, "show_results_when_ready" : False}],
]

frl_analysis_widget_list_interactive = [
    [base_metrology_path + "ow_frl_init.OWFRLInit", (0.0, 50.0), {}],
    [base_metrology_path + "ow_frl_manager_initialization.OWFRLManagerInitialization", None, {"is_automatic_run": True}],
    [base_metrology_path + "ow_frl_crop_thickness.OWFRLCropThickness", None, {"is_automatic_run": False}],
    [base_metrology_path + "ow_frl_manage_crop_thickness.OWFRLManageCropThickness", None, {"is_automatic_run": True}],
    [base_metrology_path + "ow_frl_center_image.OWFRLCenterImage", (50.0, 200.0), {"is_automatic_run": True}],
    [base_metrology_path + "ow_frl_fit_radius_dpc.OWFRLFitRadiusDPC", None, {"is_automatic_run": True}],
    [base_metrology_path + "ow_frl_do_fit.OWFRLDoFit", None, {"is_automatic_run": True}],
]

frl_analysis_logger_widget_props = [base_metrology_path + "ow_frl_logger.OWFRLLogger", (0.0, 350.0), {}]

class WavePyMenu(OMenu):

    def __init__(self):
        super().__init__(name="WavePy2")


        self.openContainer()
        self.addContainer("Imaging")
        self.addSubMenu("Create Single Grating Talbot analysis (interactive)")
        self.addSubMenu("Create Single Grating Talbot analysis (not interactive)")
        self.closeContainer()

        self.openContainer()
        self.addContainer("Coherence")
        self.addSubMenu("Create Single Grating Z Scan analysis (interactive)")
        self.addSubMenu("Create Single Grating Z Scan analysis (not interactive)")
        self.closeContainer()

        self.openContainer()
        self.addContainer("Metrology")
        self.addSubMenu("Create Fit Residual Lenses analysis (interactive)")
        self.addSubMenu("Create Fit Residual Lenses analysis (not interactive)")
        self.closeContainer()

        self.addSeparator()

        self.openContainer()
        self.addContainer("Plotter Options")
        self.addSubMenu("Set Plotter Mode: FULL")
        self.addSubMenu("Set Plotter Mode: DISPLAY ONLY")
        self.closeContainer()

        self.openContainer()
        self.addContainer("Logger Options")
        self.addSubMenu("Set Logger Mode: FULL")
        self.addSubMenu("Set Logger Mode: WARNING")
        self.addSubMenu("Set Logger Mode: ERROR")
        self.addSubMenu("Set Logger Mode: NONE")
        self.closeContainer()

    def executeAction_1(self, action):
        if _showConfirmMessage("Confirm Action", "Create Single Grating Talbot analysis (interactive)?") == QMessageBox.Yes:
            self.__create_analysis(sgt_analysis_widget_list_interactive, sgt_analysis_logger_widget_props)

    def executeAction_2(self, action):
        if _showConfirmMessage("Confirm Action", "Create Single Grating Talbot analysis (not interactive)?") == QMessageBox.Yes:
            self.__create_analysis(sgt_analysis_widget_list_not_interactive, sgt_analysis_logger_widget_props)

    def executeAction_3(self, action):
        if _showConfirmMessage("Confirm Action", "Create Single Grating Z Scan analysis (interactive)?") == QMessageBox.Yes:
            self.__create_analysis(sgz_analysis_widget_list_interactive, sgz_analysis_logger_widget_props)

    def executeAction_4(self, action):
        if _showConfirmMessage("Confirm Action", "Create Single Grating Z Scan analysis (not interactive)?") == QMessageBox.Yes:
            self.__create_analysis(sgz_analysis_widget_list_not_interactive, sgz_analysis_logger_widget_props)

    def executeAction_5(self, action):
        if _showConfirmMessage("Confirm Action", "Create Fit Residual Lenses analysis (interactive)?") == QMessageBox.Yes:
            self.__create_analysis(frl_analysis_widget_list_interactive, frl_analysis_logger_widget_props)

    def executeAction_6(self, action):
        if _showConfirmMessage("Confirm Action", "Create Fit Residual Lenses analysis (not interactive)?") == QMessageBox.Yes:
            self.__create_analysis(frl_analysis_widget_list_not_interactive, frl_analysis_logger_widget_props)

    def executeAction_7(self, action):
        self.__switch_plotter_mode(PlotterMode.FULL, "FULL")

    def executeAction_8(self, action):
        self.__switch_plotter_mode(PlotterMode.DISPLAY_ONLY, "DISPLAY ONLY")

    def executeAction_9(self, action):
        self.__switch_logger_mode(LoggerMode.FULL, "FULL")

    def executeAction_10(self, action):
        self.__switch_logger_mode(LoggerMode.WARNING, "WARNING")

    def executeAction_11(self, action):
        self.__switch_logger_mode(LoggerMode.ERROR, "ERROR")

    def executeAction_12(self, action):
        self.__switch_logger_mode(LoggerMode.NONE, "NONE")

    #################################################################
    #
    # PRIVATE METHODS
    #
    #################################################################

    def __create_analysis(self, widgets_list, logger_widget_props=None):
        nodes = []
        lower_row_position = self.__get_lower_row_position()

        for widget, position, attributes in widgets_list:
            if not position is None: position = (position[0], position[1] + lower_row_position)

            nodes.extend(self.createNewNodeAndWidget(widget_desc=self.getWidgetDesc(widget),
                                                     position=position,
                                                     attributes=attributes))
        self.createLinks(nodes)

        if not logger_widget_props is None:
            self.createNewNodeAndWidget(widget_desc=self.getWidgetDesc(logger_widget_props[0]),
                                        position=(logger_widget_props[1][0], logger_widget_props[1][1] + lower_row_position),
                                        attributes=logger_widget_props[2])

    def __get_lower_row_position(self):
        try:
            lower_row_position = max([link.sink_node.position[1] for link in self.canvas_main_window.current_document().scheme().links])
            if lower_row_position != 0.0: return 300.0 + lower_row_position
        except: return 0.0

    def __switch_plotter_mode(self, plotter_mode, plotter_name):
        existing_plotter_mode = QSettings().value("wavepy/plotter_mode", PlotterMode.FULL, type=int)
        try:
            QSettings().setValue("wavepy/plotter_mode", plotter_mode)

            for node in self.canvas_main_window.current_document().scheme().nodes:
                widget = self.canvas_main_window.current_document().scheme().widget_for_node(node)
                if isinstance(widget, WavePyInitWidget):
                    widget.initialize_plotter(replace=True)
                    widget.initialize_process_manager()

            _showInfoMessage("Plotter Mode set to: " + plotter_name + "\nRestart any analysis from Init widget to make it effective")
        except Exception as exception:
            QSettings().setValue("wavepy/plotter_mode", existing_plotter_mode)
            QMessageBox.critical(None, "Error", exception.args[0], QMessageBox.Ok)

    def __switch_logger_mode(self, logger_mode, logger_name):
        existing_logger_mode = QSettings().value("wavepy/logger_mode", LoggerMode.FULL, type=int)
        try:
            QSettings().setValue("wavepy/logger_mode", logger_mode)

            for node in self.canvas_main_window.current_document().scheme().nodes:
                widget = self.canvas_main_window.current_document().scheme().widget_for_node(node)
                if isinstance(widget, WavePyInitWidget):
                    widget.initialize_logger(replace=True)
                    widget.initialize_process_manager()

            _showInfoMessage("Logger Mode set to: " + logger_name + "\nRestart any analysis from Init widget to make it effective")
        except Exception as exception:
            QSettings().setValue("wavepy/logger_mode", existing_logger_mode)
            QMessageBox.critical(None, "Error", exception.args[0], QMessageBox.Ok)

    #################################################################
    #
    # SCHEME MANAGEMENT
    #
    #################################################################

    def getWidgetFromNode(self, node):
        return self.canvas_main_window.current_document().scheme().widget_for_node(node)

    def createLinks(self, nodes):
        previous_node = None
        for node in nodes:
            if not previous_node is None:
                link = SchemeLink(source_node=previous_node, source_channel="WavePy Data", sink_node=node, sink_channel="WavePy Data")
                self.canvas_main_window.current_document().addLink(link=link)
            previous_node = node

    def getWidgetDesc(self, widget_name):
        return self.canvas_main_window.widget_registry.widget(widget_name)

    def createNewNode(self, widget_desc, position=None):
        return self.canvas_main_window.current_document().createNewNode(widget_desc, position=position)

    def createNewNodeAndWidget(self, widget_desc, position=None, attributes={}):
        nodes = []

        nodes.append(self.createNewNode(widget_desc, position))
        widget = self.getWidgetFromNode(nodes[0])

        for attribute in attributes.keys(): setattr(widget, attribute, attributes[attribute])

        return nodes


def _showInfoMessage(message):
    _create_message_box(QMessageBox.Information, message, QMessageBox.Ok).exec_()

def _showWarningMessage(message):
    _create_message_box(QMessageBox.Warning, message, QMessageBox.Ok).exec_()

def _showCriticalMessage(message):
    _create_message_box(QMessageBox.Critical, message, QMessageBox.Ok).exec_()

def _showConfirmMessage(text, message):
    msgBox = _create_message_box(QMessageBox.Question, message, QMessageBox.Yes | QMessageBox.No)
    msgBox.setDefaultButton(QMessageBox.No)

    return msgBox.exec_()

def _create_message_box(icon, message, std_buttons):
    msgBox = QMessageBox()
    msgBox.setIcon(icon)
    msgBox.setText(message)
    msgBox.setStandardButtons(std_buttons)
    return msgBox



# #########################################################################
# Copyright (c) 2020, UChicago Argonne, LLC. All rights reserved.         #
#                                                                         #
# Copyright 2020. UChicago Argonne, LLC. This software was produced       #
# under U.S. Government contract DE-AC02-06CH11357 for Argonne National   #
# Laboratory (ANL), which is operated by UChicago Argonne, LLC for the    #
# U.S. Department of Energy. The U.S. Government has rights to use,       #
# reproduce, and distribute this software.  NEITHER THE GOVERNMENT NOR    #
# UChicago Argonne, LLC MAKES ANY WARRANTY, EXPRESS OR IMPLIED, OR        #
# ASSUMES ANY LIABILITY FOR THE USE OF THIS SOFTWARE.  If software is     #
# modified to produce derivative works, such modified software should     #
# be clearly marked, so as not to confuse it with the version available   #
# from ANL.                                                               #
#                                                                         #
# Additionally, redistribution and use in source and binary forms, with   #
# or without modification, are permitted provided that the following      #
# conditions are met:                                                     #
#                                                                         #
#     * Redistributions of source code must retain the above copyright    #
#       notice, this list of conditions and the following disclaimer.     #
#                                                                         #
#     * Redistributions in binary form must reproduce the above copyright #
#       notice, this list of conditions and the following disclaimer in   #
#       the documentation and/or other materials provided with the        #
#       distribution.                                                     #
#                                                                         #
#     * Neither the name of UChicago Argonne, LLC, Argonne National       #
#       Laboratory, ANL, the U.S. Government, nor the names of its        #
#       contributors may be used to endorse or promote products derived   #
#       from this software without specific prior written permission.     #
#                                                                         #
# THIS SOFTWARE IS PROVIDED BY UChicago Argonne, LLC AND CONTRIBUTORS     #
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT       #
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS       #
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL UChicago     #
# Argonne, LLC OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,        #
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,    #
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;        #
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER        #
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT      #
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN       #
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE         #
# POSSIBILITY OF SUCH DAMAGE.                                             #
# #########################################################################
from orangewidget import gui
from oasys.widgets import gui as oasysgui

from orangecontrib.wavepy2.util.gui.ow_wavepy_widget import WavePyWidget
from orangecontrib.wavepy2.util.wavepy_objects import OasysWavePyData


class WavePyProcessWidget(WavePyWidget):
    CONTROL_AREA_HEIGTH = 900
    CONTROL_AREA_WIDTH  = 1600

    MAX_WIDTH_NO_MAIN = CONTROL_AREA_WIDTH + 10
    MAX_HEIGHT = CONTROL_AREA_HEIGTH + 10

    inputs = [("WavePy Data", OasysWavePyData, "set_input"),]

    outputs = [{"name": "WavePy Data",
                "type": OasysWavePyData,
                "doc": "WavePy Data",
                "id": "WavePy_Data"}]

    must_clean_layout       = True
    show_results_when_ready = True

    def __init__(self, show_general_option_box=True, show_automatic_box=True, show_results_when_ready_box=True):
        super(WavePyProcessWidget, self).__init__(show_general_option_box=show_general_option_box, show_automatic_box=show_automatic_box)

        self.setFixedWidth(self.MAX_WIDTH_NO_MAIN)
        self.setFixedHeight(self.MAX_HEIGHT)

        if show_results_when_ready_box : gui.checkBox(self._general_options_box, self, 'show_results_when_ready', 'Show results when ready')
        else: self.show_results_when_ready = False

        gui.rubber(self.controlArea)

    def set_input(self, data):
        if not data is None:
            data = data.duplicate()

            self._initialization_parameters = data.get_initialization_parameters()
            self._calculation_parameters    = data.get_calculation_parameters()
            self._process_manager           = data.get_process_manager()

            if self.is_automatic_run: self._execute()

    def _execute(self):
        self.progressBarInit()

        if self.must_clean_layout: self._clear_wavepy_layout()

        self.progressBarSet(10)

        output_calculation_parameters = self._get_output_parameters()

        self.progressBarSet(90)

        self.controlArea.setFixedWidth(self.CONTROL_AREA_WIDTH)
        self.controlArea.setFixedHeight(self.CONTROL_AREA_HEIGTH)

        gui.rubber(self.controlArea)

        output = OasysWavePyData()
        output.set_process_manager(self._process_manager)
        output.set_initialization_parameters(self._initialization_parameters)
        output.set_calculation_parameters(output_calculation_parameters)

        self.progressBarSet(100)
        self.progressBarFinished()

        self.send("WavePy Data", output)

        if self.show_results_when_ready: self.show()

    def _get_output_parameters(self):
        raise NotImplementedError()

from orangecontrib.wavepy2.util.gui.ow_wavepy_widget import clear_layout
from aps.wavepy2.util import DefaultContextWidget

class WavePyProcessWidgetWithOptions(WavePyProcessWidget):

    def __init__(self, show_general_option_box=True, show_automatic_box=True, show_results_when_ready_box=True):
        super(WavePyProcessWidgetWithOptions, self).__init__(show_general_option_box=show_general_option_box, show_automatic_box=show_automatic_box, show_results_when_ready_box=show_results_when_ready_box)

        self._options_area               = oasysgui.widgetBox(self._wavepy_widget_area, "Options", addSpace=False, orientation="vertical",
                                                              width=self._get_option_area_width())
        self._lateral_wavepy_widget_area = oasysgui.widgetBox(self._wavepy_widget_area, "", addSpace=False, orientation="vertical",
                                                              width=self.CONTROL_AREA_WIDTH - self._get_option_area_width())

    def _get_option_area_width(self):
        return 200

    def _clear_wavepy_layout(self):
        clear_layout(self._lateral_wavepy_widget_area.layout())

    def _get_default_context(self):
        return DefaultContextWidget(self._lateral_wavepy_widget_area)
